# $Id: Frame.py,v 1.30 2005/12/31 15:43:54 asc Exp $

__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/ImageWidget/"
__version__    = "1.0"
__cvsversion__ = "$Revision: 1.30 $"
__date__       = "$Date: 2005/12/31 15:43:54 $"
__copyright__  = "Copyright (c) 2004-2006 Aaron Straup Cope. GNU GPL 2"

import wx
import os
import sys
import webbrowser
import thread

from   ImageWidget.Constants import *

from ImageWidget.Panel     import ImageWidgetPanel
from ImageWidget.Logger    import ImageWidgetLogger
from ImageWidget.Error     import ImageWidgetError
from ImageWidget.History   import ImageWidgetHistory
from ImageWidget.TextUtils import wrap
from ImageWidget.Filenames  import filename_history, filename_log

wx.wxConvCurrent = "utf8"

class ImageWidgetFrame (wx.Frame):

    def __init__(self, parent, ID, title,pos=IMAGEWIDGET_DEFAULT_POS,sz=IMAGEWIDGET_DEFAULT_SZ) :
        
        wx.Frame.__init__(self, parent, ID, title,
			  pos,sz)

        self.CreateStatusBar();

	#

        FileMenu = wx.Menu()

        FileMenu.Append(IMAGEWIDGET_ID_NEXT,
			IMAGEWIDGET_MENU_NEXT,
			IMAGEWIDGET_DESCRIBE_NEXT)

        FileMenu.Append(IMAGEWIDGET_ID_PAUSE,
			IMAGEWIDGET_MENU_PAUSE,
			IMAGEWIDGET_DESCRIBE_PAUSE)

        FileMenu.Append(IMAGEWIDGET_ID_RESUME,
			IMAGEWIDGET_MENU_RESUME,
			IMAGEWIDGET_DESCRIBE_RESUME)

        FileMenu.AppendSeparator()
            
        FileMenu.Append(IMAGEWIDGET_ID_EXIT,
			IMAGEWIDGET_MENU_QUIT,
			IMAGEWIDGET_DESCRIBE_QUIT)

        if "__WXMAC__" in wx.PlatformInfo:
            wx.App.SetMacExitMenuItemId(IMAGEWIDGET_ID_EXIT)

        FileMenu.Enable(IMAGEWIDGET_ID_RESUME,
			False)

        #

	EditMenu = wx.Menu()

	EditMenu.Append(IMAGEWIDGET_ID_MAXWIDTH,
			IMAGEWIDGET_MENU_MAXWIDTH,
			IMAGEWIDGET_DESCRIBE_MAXWIDTH)

	EditMenu.Append(IMAGEWIDGET_ID_MAXHEIGHT,
			IMAGEWIDGET_MENU_MAXHEIGHT,
			IMAGEWIDGET_DESCRIBE_MAXHEIGHT)

	#

        ToolsMenu = wx.Menu()

        ToolsMenu.Append(IMAGEWIDGET_ID_INFO,
			 IMAGEWIDGET_MENU_INFO,
	                 IMAGEWIDGET_DESCRIBE_INFO)

	ToolsMenu.Append(IMAGEWIDGET_ID_BROWSEROPEN,
			 IMAGEWIDGET_MENU_BROWSEROPEN,
			 IMAGEWIDGET_DESCRIBE_BROWSEROPEN)

	ToolsMenu.Append(IMAGEWIDGET_ID_COPYURL,
			 IMAGEWIDGET_MENU_COPYURL,
			 IMAGEWIDGET_DESCRIBE_COPYURL)
	
	ToolsMenu.AppendSeparator()

        ToolsMenu.AppendCheckItem(IMAGEWIDGET_ID_LOG,
                                  IMAGEWIDGET_MENU_LOG,
                                  IMAGEWIDGET_DESCRIBE_LOG)
	
	#

        HelpMenu = wx.Menu()

        HelpMenu.Append(IMAGEWIDGET_ID_ABOUT,
			IMAGEWIDGET_MENU_ABOUT,
	                IMAGEWIDGET_DESCRIBE_ABOUT)

        if "__WXMAC__" in wx.PlatformInfo:
            wx.App.SetMacAboutMenuItemId(IMAGEWIDGET_ID_ABOUT)

	#

        menuBar = wx.MenuBar()

        menuBar.Append(FileMenu,
		       IMAGEWIDGET_MENUBAR_FILE)

	menuBar.Append(EditMenu,
		       IMAGEWIDGET_MENUBAR_EDIT)

        menuBar.Append(ToolsMenu,
		       IMAGEWIDGET_MENUBAR_TOOLS)

        menuBar.Append(HelpMenu,
		       IMAGEWIDGET_MENUBAR_HELP)

        self.SetMenuBar(menuBar);

        self.Panel = ImageWidgetPanel(self, -1)

        #

        name = self.GetTitle()
        self.__History = ImageWidgetHistory(filename_history(name), name)

        #
        
        self.Image   = None
        self.__Next  = None
	self.Caption = None
	self.Logger  = None
	self.Data    = {}
        
	self.Paused   = False
        self.ShowNext = False
        self.Die      = False

        #
        
        self.next_lock  = thread.allocate_lock()

        #
        
        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_MAXWIDTH,
		    self.SetMaxWidth)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_MAXHEIGHT,
		    self.SetMaxHeight)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_NEXT,
		    self.Next)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_PAUSE,
		    self.Pause)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_RESUME,
		    self.Resume)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_INFO,
		    self.Info)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_BROWSEROPEN,
		    self.OpenInBrowser)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_COPYURL,
		    self.CopyURLToClipboard)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_ABOUT,
		    self.About)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_EXIT,
		    self.Quit)

        wx.EVT_MENU(self,
		    IMAGEWIDGET_ID_LOG,
		    self.ToggleLogger)

        wx.EVT_CLOSE(self,
		     self.Quit)
        
        wx.EVT_MENU_CLOSE(self,
			  self.SetCaption)

	wx.EVT_KEY_DOWN(self.Panel,
			self.KeyDown)
    
	wx.EVT_LEFT_DOWN(self.Panel,self.OnLeftDown)

    #
    #
    #
    
    def OnLeftDown (self,event):
	if event.CmdDown():
	    self.OpenInBrowser(event)

    #
    #
    #
    
    def Setup (self) :
        self.Show(True)

	self.Offset = (self.GetSize())[1] - (self.Panel.GetSize())[1]
        self.Panel.SetFocus()

	#

	cfg = self.Config()
	ns  = cfg.nsCfg()
	log = cfg.GetPref(ns[IMAGEWIDGET_CFG_LOGGING])

	menu  = self.GetMenuBar();
	index = menu.FindMenu(IMAGEWIDGET_MENUBAR_TOOLS)
	tools = menu.GetMenu(index);

	if log == '1' :
            tools.Check(IMAGEWIDGET_ID_LOG,True)
            self.SetLogger(True)
        else :
            tools.Check(IMAGEWIDGET_ID_LOG,False)

    #
    #
    #
    
    def Yield (self) :
        
        # This causes Windows to hang
        # FreeBSD hangs without it...

        if sys.platform == "nt":
            pass
        else:
	    wx.SafeYield(self,True)

    #
    #
    #
    
    def ReportError (self,msg,exception=None):

        if hasattr(msg,'encode') :
            log = msg.encode('ascii','replace')
            self.Log("error","%s (%s)" % (log,exception))
        else :
            self.Log("error","%s (%s)" % (msg,exception))

        # print "msg %s (%s)" % (msg,type(msg))
            
	if msg:
	    self.AssignStatusText(msg)
	else: 
	    self.AssignStatusText("")

        self.Yield()

    #
    #
    #
    
    def ReportActivity (self,msg):
	if msg:
	    msg = "[%s]" % self.PrepareStatusText(msg)
	    self.AssignStatusText(msg)
	else: 
	    self.AssignStatusText("")

        self.Yield()

    #
    #
    #

    def OnNext (self, event) :
        self._parent.OnNext(event)

    #
    #
    #
    
    def Next (self, event) :

        self.OnNext(event)
        
        if self.Paused == True :
            self.Resume(event)
        else :
            self.ShowNext = True
        
    #
    #
    #

    def OnPause (self, event) :
        self._parent.OnPause(event)

    #
    #
    #
    
    def Pause (self,event):

        self.OnPause(event)
        
        if self.Paused == True:
            return True
        
        self.Paused = True
        self.ReportActivity(IMAGEWIDGET_MSG_PAUSED)

	self.GetMenuBar().Enable(IMAGEWIDGET_ID_PAUSE,False)
	self.GetMenuBar().Enable(IMAGEWIDGET_ID_RESUME,True)
    
    #
    #
    #
    
    def Resume (self,event):

        if self.Paused == False:
            return True
        
        self.Paused = False

	self.GetMenuBar().Enable(IMAGEWIDGET_ID_PAUSE,True)
	self.GetMenuBar().Enable(IMAGEWIDGET_ID_RESUME,False)

    #
    #
    #
    
    def OpenInBrowser (self,event) :

	if self.Data == None or len(self.Data) == 0:
	    self.ReportError(IMAGEWIDGET_ERR_IMAGEINFO)
	    return False

        try :
            self.OpenURLInBrowser(self.Data['url'])        
        except Exception, e :
            raise ImageWidgetError(e)
            
    #
    #
    #
    
    def OpenURLInBrowser (self, url) :
	try :
            # Something about calling wx.LaunchDefaultBrowser
            # causes two windows to be open, where the first
            # tries to load 'file://%3c/'
            # wx.LaunchDefaultBrowser(url)   
            webbrowser.open(url)
	except Exception, e :
            raise ImageWidgetError(IMAGEWIDGET_ERR_BROWSEROPEN,e)
        
    #
    #
    #
    
    def Info (self,event):

	if self.Data == None or len(self.Data) == 0:
	   msg = IMAGEWIDGET_ERR_IMAGEINFO
	else:

            title = IMAGEWIDGET_INFO_TITLE
            
            msg  = self.Data["creator"]
            msg += "\n"
            
            if len(self.Data["title"]) :
                msg += "\"" + self.Data["title"] + "\""
                msg += "\n\n"

                title = self.Data["title"]
                
            if len(self.Data["description"]) :
                msg += self.Data["description"]
                msg += "\n"

            if len(self.Data["location"]) :
                msg += "\n"
                msg += self.Data["date"]

            if len(self.Data["date"]) :

                if len(self.Data["location"]) :
                    msg += u"\N{HORIZONTAL BAR}"
                else :
                    msg += "\n"

                msg += self.Data["date"]
                msg += "\n"

            if len(self.Data["license"]) :
                msg += "\n"
                msg += self.Data['license']

            dlg = wx.MessageDialog(self,
                                   msg,
                                   title,
                                   wx.OK)

            dlg.ShowModal();
            dlg.Destroy();

    #
    #
    #
    
    def KeyDown (self,event):
        
	if event.CmdDown():
            keycode = event.GetKeyCode()
        else:
            event.Skip()
            return True
        
        if keycode <= 256:
            keyname = chr(keycode)
        else:
            event.Skip()
            return True
	
	# Because Cmd-H hides the application
	# in happy OS X land...

	if event.ShiftDown() :
	    if keyname == "H":
		self.SetMaxHeight(event)
	    elif keyname == "W":
		self.SetMaxWidth(event)
	    else:
		event.Skip()

	    return True

	#

	if keyname == "A":
            self.About(event)
	elif keyname == "C":
            self.CopyURLToClipboard(event)            
        elif keyname == "I":
            self.Info(event)
        elif keyname == "N" :
            self.Next(event)
        elif keyname == "L":
            self.ToggleLogger(event)
        elif keyname == "O":
            self.OpenInBrowser(event)
        elif keyname == "P":
            self.Pause(event)
        elif keyname == "R":
            self.Resume(event)
        elif keyname == "Q":
            self.Quit(event)
        else:
            event.Skip()
	
	return True

    #
    #
    #
    
    def About (self,event,title=IMAGEWIDGET_APP_NAME,body=IMAGEWIDGET_APP_DESCRIPTION) :
        dlg = wx.MessageDialog(self,
			       body,
			       title,
			       wx.OK)
        dlg.ShowModal()
        dlg.Destroy()

    #
    #
    #
    
    def SetMaxWidth (self,event):

	cfg = self.Config()
	ns  = cfg.nsCfg()

	default = cfg.GetPref(ns[IMAGEWIDGET_CFG_MAXWIDTH])	
	prompt  = wrap(IMAGEWIDGET_CFG_MAXWIDTH_PROMPT,50)

        max = wx.GetTextFromUser(prompt,
				 IMAGEWIDGET_CFG_MAXWIDTH_TITLE,
				 str(default))

        if max == "":
            return

        try:
            max = int(max)
        except:
            errmsg = IMAGEWIDGET_CFG_MAXWIDTH_ERROR % str(max)
            self.ReportError(errmsg)
            return False

	cfg.SetPref(ns[IMAGEWIDGET_CFG_MAXWIDTH],max)
	cfg.Save()

    #
    #
    #
    
    def SetMaxHeight (self,event):

	cfg = self.Config()
	ns  = cfg.nsCfg()

	default = cfg.GetPref(ns[IMAGEWIDGET_CFG_MAXHEIGHT])	
	prompt  = wrap(IMAGEWIDGET_CFG_MAXHEIGHT_PROMPT,50)

        max = wx.GetTextFromUser(prompt,
				 IMAGEWIDGET_CFG_MAXHEIGHT_TITLE,
				 str(default))

        if max == "":
            return

        try:
            max = int(max)
        except:
            errmsg = IMAGEWIDGET_CFG_MAXHEIGHT_ERROR % str(max)
            self.ReportError(errmsg)
            return False

	cfg.SetPref(ns[IMAGEWIDGET_CFG_MAXHEIGHT],max)
	cfg.Save()

    #
    #
    #
    
    def SavePosition (self) :
	(x,y) = self.GetPosition()

	cfg = self.Config()
	ns  = cfg.nsCfg()

	cfg.SetPref(ns[IMAGEWIDGET_CFG_POSX],x)
	cfg.SetPref(ns[IMAGEWIDGET_CFG_POSY],y)
	cfg.Save()

    #
    #
    #

    def OnQuit (self, event) :
        self._parent.OnQuit(event)

    #
    #
    #
    
    def Quit (self, event):
        self.OnQuit(event)
        
	self.SavePosition()
	self.Panel.Destroy()
        self.Destroy()
    
    #
    #
    #

    def CollectNextImage (self,threaded=False) :

        self.next_lock.acquire()

        if not self.__Next == None :
            self.next_lock.release()
            return True

        try :
            (data,wx_image) = self.FetchImage(False)
        except Exception, e :
            self.next_lock.release()

            if threaded == True :
                return False
            else :
                raise ImageWidgetError(e)

        self.__Next = {}
        self.__Next['data']  = data
        self.__Next['image'] = wx_image

        self.next_lock.release()
        return True
        
    #
    #
    #
    
    def FetchImage (self,verbose=True) :
                    
	try :
	    parent = self._parent
            data   = parent.GetImage()
	except Exception, e :
            raise ImageWidgetError("%s (%s) " % (IMAGEWIDGET_ERR_POLLING,e))

	#

        if verbose == True :
            self.ReportActivity(IMAGEWIDGET_MSG_PROCESSING)

        try:
            wx_image = wx.ImageFromStream(data['image'])
        except Exception, e:
            raise ImageWidgetError("%s (%s) " % (IMAGEWIDGET_ERR_STREAMIFY,e))

        return (data,wx_image)

    #
    #
    #

    def OnDisplay (self) :
        self._parent.OnDisplay()
                    
    #
    #
    #
    
    def DisplayRandom (self) :
        
        self.OnDisplay()
        
        self.ReportActivity(IMAGEWIDGET_MSG_POLLING)
        
        if self.__Next == None :
            try :
                (data,wx_image) = self.FetchImage()
            except Exception, e:
                self.ReportError(IMAGEWIDGET_ERR_FETCHIMAGE,e)
                return False
            self.Data = data
        else :
            self.Data = self.__Next['data']
            wx_image  = self.__Next['image']

            self.next_lock.acquire()
            self.__Next = None
            self.next_lock.release()

        #
        
	try :
	    (maxheight,maxwidth) = self._parent.GetMaxDimensions()

	    if maxheight or maxwidth :
		image = self.Panel.ScaleImage(wx_image,maxheight,maxwidth)

	except Exception, e:
            self.ReportError(IMAGEWIDGET_ERR_DISPLAY,e)
            return False
        
        try:
            self.Panel.Display(wx_image)
        except Exception, e:
            self.ReportError(IMAGEWIDGET_ERR_DISPLAY,e)
            return False

	#
	
	try:
	    (img_x,img_y)  = wx_image.GetSize()
	except Exception, e:
	    self.ReportError(IMAGEWIDGET_ERR_GETSIZE,e);
	    return False;

	sz = wx.Size(img_x + (IMAGEWIDGET_DIM_BORDER * 2),
		     img_y + (IMAGEWIDGET_DIM_BORDER * 2) + self.Offset)

	self.SetSize(sz)

	#

	caption = self.Data["creator"]

	if self.Data["title"]:
	    caption += ", "
	    caption += self.Data["title"]

	#

	chars        = int(wx_image.GetWidth() * IMAGEWIDGET_STATUSTEXT)
	self.Caption = self.PrepareStatusText(caption,chars)

	self.SetCaption()

        # self.__History.Add(self.Data)
        thread.start_new_thread(self.__History.Add,(self.Data,))
        return True
    
    #
    #
    #
    
    def SetCaption (self,event=None):
	if self.Caption:
	   self.AssignStatusText(self.Caption)
	else:
	   self.AssignStatusText("")

    #
    #
    #
    
    def AssignStatusText(self,txt="") :

	self.SetStatusText(txt)

    #
    # Make sure that the text will fit in
    # the status bar - txt is a unicode object
    #
    
    def PrepareStatusText (self,txt,max=0):

	(txt_x,txt_y) = self.GetTextExtent(txt)

	if max == 0:
	   max = int(self.GetSize()[0] * IMAGEWIDGET_STATUSTEXT)

	if txt_x > max:

	   while txt_x > max:

		# please make the python keeners
		# shut the fuck up about how great
		# their language is. my kingdom for
		# a substring function...

		txt = txt[0:len(txt) - 1]

		# just in case

		if len(txt) == 1:
		   txt = u""
		   break

	        (txt_x,txt_y) = self.GetTextExtent(txt)

	   txt += u"..."

	#

	return txt

    #
    #
    #
    
    def ToggleLogger (self,evt) :

        bool = None
        
        if self.Logger == None :
            bool = True
        else :
            bool = False

        return self.SetLogger(bool)

    #
    #
    #
    
    def Log (self,level,msg) :
        if self.Logger == None :
            return

        self.Logger.dispatch(level,msg)

    #
    #
    #
    
    def SetLogger (self,bool) :

        if bool == False :
            self.ReportActivity(IMAGEWIDGET_MSG_NOLOG)
            self.Log("info",IMAGEWIDGET_MSG_NOLOG)
            self.Logger = None
            pref        = 0
        else :

            self.Logger = ImageWidgetLogger(filename_log(self.GetTitle()),__name__)
            self.ReportActivity(IMAGEWIDGET_MSG_LOG)
	    pref = 1

        #
        
        self.GetMenuBar().Check(IMAGEWIDGET_ID_LOG,bool)

	cfg = self.Config()
	ns  = cfg.nsCfg()
	log = cfg.SetPref(ns[IMAGEWIDGET_CFG_LOGGING],pref)

        return True

    #
    #
    #
    
    def CopyURLToClipboard (self, event) :

	if self.Data == None or len(self.Data) == 0:
	    self.ReportError(IMAGEWIDGET_ERR_IMAGEINFO)
	    return False

        url = wx.TextDataObject()
        url.SetText(self.Data['url'])
        
        cb = wx.TheClipboard
        
        if cb.Open():
            cb.SetData(url)
            cb.Close()
        else :
            self.ReportError(IMAGEWIDGET_ERR_CLIPBOARDOPEN)
            
    #
    #
    #

    def Config (self) :
	return self._parent.Config()
